; ===============================================================
; FUNCTION: to move data from the panel to the program
; ===============================================================

(defun gDialParms()                            ; Invoked after
  (setq lat(atof(get_tile "dLat")))            ; the DRAW
  (setq lng(atof(get_tile "dLng")))            ; button is used
  (setq ref(atof(get_tile "dRef")))
)


; ===============================================================
; FUNCTION: to get user parameters and display a dial plate
; ===============================================================

; This is a vertical dial program written in AutoCAD LISP
; This is intended for ProgeCAD from ProgeSOFT CAD system
; This is a skeleton only, but the data for hour line angles validates
;
; KNOWN BUG:  hours between 7 and 5pm are shown, this is because 
;             tan fails when angles approach 90 degrees in this version
;             of LISP. And there is a d.long test to limit it.
;
; HISTORY:    Jan 6, 2008 LISP uses Polish notation.
;             Mostly like an h-dial but things reversed
;             This is NOT designed for declining dials
;             Jan 6 2008 1304 fixed sense of hour lines
;
; AUTHOR:     Simon Wheaton-Smith FRI, MBCS, CITP  Jan 26, 2008 

(defun C:vDial()

  ; dont echo graphical commands to the log
  (setvar "cmdecho" 0)

  ; ------------  Load the v-dial form
  (setq dcl_id (load_dialog "vDialForm.dcl"))

  ; ------------  If the form wasnt there them do bad things
  (if (not (new_dialog "vDialForm" dcl_id))
    (progn
     (alert "The vDialForm.DCL file could not be loaded!")
      (exit)
    )
  )

  ; ------------  do this set of commands when something happens
  (action_tile "draw" "(setq ddiag 2)(gDialParms)(done_dialog)")
  (action_tile "cancel" "(setq ddiag 1)(done_dialog)")

  ; ------------  Display the panel
  (start_dialog)

  ; ------------  Unload the panel
  (unload_dialog dcl_id)

  ; ------------  If Cancel
  (if(= ddiag 1)                               ; LISP uses Polish
    (princ "\n ... no dial will be generated") ; notation, get over it.
  )

  ; ------------  If draw
  ;[lat]     is a design latitude
  ;[lng]     is a design longitude
  ;[ref]     is a reference longitude
  ;[dlong]   is longitude correction
  ;[clat]    is the COSINE of the latitude
  ;[thra]    is the tan of the hour angle

  ( if(= ddiag 2)
       (progn
         (grclear)
         (princ "\n... V dial will be generated v.200801070840")
         (princ "\n")
         (setq dlong (- lng ref))
         (princ "Latitude:    ")       (princ lat)        (princ "\n")
         (princ "Longitude:   ")       (princ lng)        (princ "\n")   
         (princ "   ref.long: ")       (princ ref)        (princ "\n")
         (princ "   dif.long: ")       (princ dlong)      (princ "\n")


         ; keep d.long to within 15 degrees     
         (setq adlong (abs dlong))             ; get absolute long and ref
         ( if ( > adlong 14 )                  ; difference
              (progn
                  (alert "Longitude and reference not to exceed one time zone")
                  (exit)
              )
         )

         ;
         ; CONSTANT TRIGONOMETRIC FUNCTION FOR LATITUDE 
         ;
         ; get SINE(LATITUDE)
         ;     Convert degrees to radians
         ;     Rad = (n * 2 * 3.1416) / 360
         (setq latR (* lat  2))                ; LISP uses Polish
         (setq latR (* latR pi))               ; notation, get over it. 
         (setq latR (/ latR 360))                
         (setq clat (cos latR))
         ;(princ "cos(lat):    ")       (princ clat)       (princ "\n") 

         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

         ; draw a two part box - left side
         ;         x    y     x    y
         (grdraw '(50   50) '(250  50)  -1)
         (grdraw '(50   50) '( 50 250)  -1) 
         (grdraw '(250 250) '( 50 250)  -1) 
         (grdraw '(250 250) '(250  50)  -1) 

         ; draw a two part box - right side
         ;         x    y     x    y
         (grdraw '(250  50) '(450  50)  -1)
         ;(grdraw '(250  50) '(250 250)  -1) 
         (grdraw '(450 250) '(250 250)  -1) 
         (grdraw '(450 250) '(450  50)  -1) 

         ; The dial center is x=250 y=250
         ; the x and limits are + and - 200

         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         ; MAIN HOUR LINE LOOP
         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         ;[cx]   is center of dial plate x 
         ;[cy]   is center of dial plate y     
         (setq cx 250)
         (setq cy 250)

         (setq ctr 6)                    
         (while (< ctr 19)    

                ; for ease of reading separate out noon
                ( if(= ctr 12)
                  (princ " \n")
                )
            
                ; SHOW THE HOUR
                (princ " HR:")
                (princ ctr) 
                ; SHOW THE ANGLE IN DEGREES 
                (setq hr (- ctr 12))
                ;  (princ "  hrFr12:")
                ;  (princ hr) 
                (setq hra (* hr 15))
                ;  (princ "  hra:")
                ;  (princ hra) 
                (setq lha (- hra dlong))
                ;  (princ "  lha:")
                ;  (princ lha) 

                ; ====================================================
                ; we have the sun's local  hour angle angle in degrees
                ;    now we need the hour line angle itself
                ; ====================================================

                ; [tlha] is the tangent of the local hour angle                
                ; get TANGENT(HOUR ANGLE)
                ; Convert degrees to radians
                ;    Rad = (n * 2 * 3.1416) / 360
                (setq tlha (* lha  2))         ; get lha times 2       
                (setq tlha (* tlha pi))        ; then times 3.1416       
                (setq tlha (/ tlha 360))       ; divided by 360 

                ; - - - - - - - - - - - - - - - - - - - - - - - 
                ; LISP TAN returns bad data on tan(radians(90))
                ; so we set flag=1 meaning ok to do a tan, flag=0 not ok 
                (setq flag 1)                  ; default is ok
                ( if (= lha -90) 
                     (setq flag 0)
                )
                ( if (= lha  90) 
                     (setq flag 0)
                )
                ; before doing a tan, check and see if tan will work 
                (if (= flag 1)       
                    (setq tlha (tan tlha))
                )
                ; - - - - - - - - - - - - - - - - - - - - - - - 


                (if (= flag 1)
                (progn
                ; ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                    ; print the real or false tan (flase is one we made up)
                    ;(princ "  tan(lha):")          (princ tlha)  

                    ; the guts of this part of the program is here
                    ; [thrlan] is the tangent of the hour line angle
                    ;
                    ; F O R M U L A    F O R    H O U R    L I N E    A N G L E
                    ;
                    ; hrLnAng = atan (  cos(lat) * tan (lha)  )	
                    ;                       slat       tlha
                    ;
                    ; F O R M U L A    F O R    H O U R    L I N E    A N G L E
                    ;
                    (setq thrlan ( * clat tlha))
                    ; (princ "  tan(hr.ln.ang):")    (princ thrlan)  

                    ; [hrlan] is the hour line angle in radians
                    (setq hrlan ( atan thrlan))
                    ; (princ "  atan(hr.ln.ang)[rad]:")  (princ hrlan)     
                    ; >>>>> [hrlan] is the hour line angle in radians for polar coordinates 
    
                    ; [hrlanD] is the the hour line angle in degrees
                    (setq hrland ( / hrlan 2))
                    (setq hrland ( / hrland pi))
                    (setq hrland ( * hrland 360))
                    (princ "  hour line angle[deg]")   (princ hrland)      

                    ; ABOVE FIGURES have been validated for 32.57 lat, 108.2 lng, ref 105

                    ; now we need to convert the angle in radians >>> hrlan
                    ; to coordinates for line drawing, with dial center at >>> x=250 y=50
                    (setq Pt0 '(250 250))
                    ; i.e.      cx  cy

                    (setq hrlan (* +1 hrlan))

                    (setq rotation (- hrlan 1.571))
                    ; 1.570796 is 90 degrees in radians, and LISP here needs a 90 shift
                    (setq pt1  (polar Pt0(+ rotation) (* 200)))

                    ; dont show hour lines off the plate area
                    (setq vhr ctr)
                    ( if(> lha -90)
                        (progn
                        ( if(< lha 90)
                            (progn
                                 (command "pline" Pt0 Pt1 "")  
                                 (command "text" pt1  8   0.0 vhr )            
                            )
                        )
                        )
                    )
                ; ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                ) ; end progn for if flag=1
                ) ; end if flag = 1
             
                ; =======================
                ; BUMP HOUR FOR NEXT LOOP
                ; =======================
                ; for ease of reading separate out noon
                ( if(= ctr 12)
                  (princ " \n")
                )
                (setq ctr(+ ctr 1)) 
                (princ "\n")             
         ) 
         (princ "\n")


       )
  )

  (princ)

)  
