; ===============================================================
; FUNCTION: to move data from the panel to the program
; ===============================================================

(defun gDialParms()                            ; Invoked after
  (setq lat(atof(get_tile "dLat")))            ; the DRAW
  (setq lng(atof(get_tile "dLng")))            ; button is used
  (setq ref(atof(get_tile "dRef")))            ; lat, lng, ref
  (setq dec(atof(get_tile "ddec")))            ; and declination
)


; ===============================================================
; FUNCTION: to get user parameters and display a dial plate
; ===============================================================

; This is a vertical decliner dial program written in AutoCAD LISP
; This is a skeleton program
;
; KNOWN BUG:  hours between 7 and 5pm are shown, this is because 
;             tan fails when angles approach 90 degrees in this version
;             of LISP. And there is a d.long test to limit it.
;
; KNOWN BUG:  some hour lines may need reciprocating, this prints a
;             note to that effect at the end.
;
; HISTORY:    Jan 6, 2008 LISP uses Polish notation.
;             Mostly like a v-dial but things reversed are more complex
;              
; AUTHOR:     Simon Wheaton-Smith FRI, MBCS, CITP Jan 26, 2008

(defun C:vdec()

  ; dont echo graphical commands to the log
  (setvar "cmdecho" 0)

  ; ------------  Load the v-dec-dial form
  (setq dcl_id (load_dialog "vdecForm.dcl"))

  ; ------------  If the form wasnt there 
  (if (not (new_dialog "vdecForm" dcl_id))
    (progn
     (alert "The vdecForm.DCL file could not be loaded")
      (exit)
    )
  )

  ; ------------  do this set of commands when something happens
  (action_tile "draw"   "(setq ddiag 2)(gDialParms)(done_dialog)")
  (action_tile "cancel" "(setq ddiag 1)(done_dialog)")

  ; ------------  Display the panel
  (start_dialog)

  ; ------------  Unload the panel
  (unload_dialog dcl_id)

  ; ------------  If Cancel
  (if(= ddiag 1)                               ; LISP uses Polish
    (princ "\n ... no dial will be generated") ; notation, get over it.
  )

  ; ------------  If Draw
  ;
  ; R A W    D A T A
  ;
  ;[lat]     is a design latitude
  ;[lng]     is a design longitude
  ;[ref]     is a reference longitude
  ;[dec]     is dial declination with          ; -SW and +SE
  ;
  ; S I M P L E    D E R I V E D    D A T A
  ;
  ;[dlong]   is longitude correction
  ;
  ; C O M P L E X    D E R I V E D    D A T A
  ;
  ;     sd  = atan( sin(dec) / tan(lat) )
  ;
  ;     sh  = asin( cos(lat) * cos(dec) )
  ;
  ;     hla = atan(cos(lat)/(cos(dec) cot(ha) + sin(dec) sin(lat))
  ;
  ;[slat]    is the SINE of the latitude
  ;[clat]    is the COSINE of the latitude
  ;[tlat]    is the TANGENT of the latitude
  ;[sdec]    is the SINE of the wall declination
  ;[cdec]    is the COSINE of the wall declination
  ;[thra]    is the tan of the hour angle


  ( if(= ddiag 2)
       (progn
         (grclear)
         (princ "\n... V dial will be generated v.200801070845")
         (princ "\n")
         (setq dlong (- lng ref))
         (princ "Latitude:    ")       (princ lat)        (princ "\n")
         (princ "Longitude:   ")       (princ lng)        (princ "\n")   
         (princ "   ref.long: ")       (princ ref)        (princ "\n")
         (princ "   dif.long: ")       (princ dlong)      (princ "\n")
         (princ "Declination: ")       (princ dec)        (princ "   and faces: ")
         ( if (< dec 0)
              (progn
                 (princ "SW *\  gnomon is to right:east")
                 (princ "\n")
              )
         )
         ( if (> dec 0)
              (progn
                 (princ "SE /*  gnomon is to left:west")
                 (princ "\n")
              )
         )
         ; we will flip the dec sign in a few lines however

         ; keep d.long to within 15 degrees     
         (setq adlong (abs dlong))             ; get absolute long and ref
         ( if ( > adlong 14 )                  ; difference
              (progn
                  (alert "Longitude and reference not to exceed one time zone")
                  (exit)
              )
         )

         ; sd  = atan( sin(dec) / tan(lat) )
         ;
         ; sh  = asin( cos(lat) * cos(dec) )
         ;
         ; hla = atan(cos(lat)/(cos(dec) cot(ha) + sin(dec) sin(lat))
          

         ; CONSTANT TRIGONOMETRIC FUNCTIONS 
         ;
         ; get trig_function(parameter)
         ;
         ;     Convert degrees to radians
         ;     Rad = (n * 2 * 3.1416) / 360
         ;

         (setq latR (* lat  2))                ; LISP uses Polish
         (setq latR (* latR pi))               ; notation, get over it. 
         (setq latR (/ latR 360)) 
         ;               
         (setq slat (sin latR))
         ;(princ "sin(lat):    ")       (princ slat)       (princ "\n") 

         (setq clat (cos latR))
         ;(princ "cos(lat):    ")       (princ clat)       (princ "\n") 

         (setq tlat (tan latR))
         ;(princ "tan(lat):    ")       (princ tlat)       (princ "\n") 

         ; DECLINATION of the wall
         (setq dec ( * -1 dec ))       ; invert sign bacause of our convention
         (setq decR (* dec  2))                
         (setq decR (* decR pi))               
         (setq decR (/ decR 360)) 
         ;               
         (setq sdec (sin decR))
         ;(princ "sin(declination):    ")       (princ sdec)       (princ "\n") 
         ;
         (setq cdec (cos decR))
         ;(princ "cos(declination):    ")       (princ cdec)       (princ "\n") 


         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

         ; draw a two part box - left side
         ;         x    y     x    y
         (grdraw '(50   50) '(250  50)  -1)
         (grdraw '(50   50) '( 50 250)  -1) 
         (grdraw '(250 250) '( 50 250)  -1) 
         (grdraw '(250 250) '(250  50)  -1) 

         ; draw a two part box - right side
         ;         x    y     x    y
         (grdraw '(250  50) '(450  50)  -1)
         (grdraw '(450 250) '(250 250)  -1) 
         (grdraw '(450 250) '(450  50)  -1) 

         ; The dial center is x=250 y=250
         ; the x and limits are + and - 200

         (princ "\n... SD and SH Angle data follows\n")
         ;
         ; sd  = atan( sin(dec) / tan(lat) )
         ;
         (setq sdr (/ sdec tlat))
         (setq sdr (atan sdr))
         (setq sdd ( / sdr 2))
         (setq sdd ( / sdd pi))
         (setq sdd ( * sdd 360))    
         (setq sdd ( * sdd -1))       ; to make consistent with DeltaCAD sign
         (princ "   SD deg:    ")       (princ sdd)       (princ "\n") 
         ;
         ; sh  = asin( cos(lat) * cos(dec) )
         ;
         (setq shr (* cdec clat))
         ;      LISP has no ASIN function so we do it ourselves:  asn = Atn(n/Sqr(1-n*n)) 
                ( if (> (abs shr) 0.99999 )
                  (progn
                  (setq sh 1)
                  ;(setq sh (* shr sign( shr )) <<<< future code to get left/right of SD
                  )
                )
                ( if (< (abs shr) 1 )
                  (progn
                  (setq sh (* shr shr))
                  (setq sh (- 1 sh))
                  (setq sh (sqrt sh))
                  (setq sh (/ shr sh))               
                  (setq sh (atan sh))               
                  )
                )
                (setq shr sh)        ; sh is ASIN(clat cdec) in radians
         (setq shd ( / shr 2))
         (setq shd ( / shd pi))
         (setq shd ( * shd 360))    
         (princ "   SH deg:    ")       (princ shd)       (princ "\n") 
         ; ABOVE FIGURES for SD and SH have been validated  


         (princ "\n... Hour Line Angle data follows\n")

         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         ; MAIN HOUR LINE LOOP                              
         ; >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
         ;[cx]   is center of dial plate x 
         ;[cy]   is center of dial plate y     
         (setq cx 250)
         (setq cy 250)

         (setq ctr 6)                    
         (while (< ctr 19)    

                ; for ease of reading separate out noon
                ( if(= ctr 12)
                  (princ " \n")
                )
            
                ; SHOW THE HOUR
                (princ " HR:")
                (princ ctr) 
                ; SHOW THE ANGLE IN DEGREES 
                (setq hr  (- ctr 12))
                (setq hra (* hr  15))
                (setq lha (- hra dlong))
                ;(princ "  lha:")              (princ lha)        (princ "   ")


                ; ====================================================
                ; we have the sun's local hour angle angle in degrees
                ; ====================================================

                ; [ tlha ] is the tangent of the local hour angle [ lha ]               
                ;          get TANGENT(HOUR ANGLE)
                ;          Convert degrees to radians = (n * 2 * 3.1416) / 360
                ;
                (setq tlha (* lha  2))         ; get lha times 2       
                (setq tlha (* tlha pi))        ; then times 3.1416       
                (setq tlha (/ tlha 360))       ; divided by 360 
                ;(princ "  lhaR:")             (princ tlha)        (princ "   ")

                ; - - - - - - - - - - - - - - - - - - - - - - - 
                ; LISP TAN returns bad data on tan(radians(90))
                ; so we set flag=1 meaning ok to do a tan, flag=0 not ok 
                (setq flag 1)                  ; default is ok
                ( if (= lha -90) 
                     (setq flag 0)
                )
                ( if (= lha  90) 
                     (setq flag 0)
                )
                ; before doing a tan, check and see if tan will work 
                (if (= flag 1) 
                    (progn      
                    ; get cotan(lha) because that is what is needed below
                    (setq tlha (tan tlha))
                    (setq cotlha (/ 1 tlha))
                    ;(princ " tlha:") (princ tlha) (princ " cttlha:") (princ cotlha)
                    )
                )
                ; - - - - - - - - - - - - - - - - - - - - - - - 


                (if (= flag 1)                 ; dont do this if TAN(lha) = 0
                (progn                         ; because div 0 is invalid
                ; ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~


                    ; the guts of this part of the program is here
                    ;
                    ; F O R M U L A    F O R    H O U R    L I N E    A N G L E
                    ;
                    ; line      = atan(cos(lat)  /  (cos(dec) * cot(ha)  +  sin(dec) * sin(lat))
                    ; angle            clat          cdec       cotlha      sdec       slat
                    ;                                --------bbb-------     --------ccc---------
                    ;                                --------------------ddd--------------------
                    ;                  ---------------------------aaa---------------------------
                    ;             ---------------------------hrlan------------------------------
                    ;
  		    ;                                       note:  cot = 1/tan
                    ;
                    ; F O R M U L A    F O R    H O U R    L I N E    A N G L E
                    ;                        

                    (setq ccc    (* sdec slat))
                    (setq bbb    (* cdec cotlha))  ; *** or *** (setq bbb (/ cdec tlha))
                    (setq ddd    (+ bbb  ccc ))
                    (setq aaa    (/ clat ddd ))
                    (setq hrlan  ( atan aaa  ))    ; [hrlan] is the hour line angle in radians
                    ;    [hrlan] is the hour line angle in radians for polar coordinates 
    

                    ; [hrlanD] is the the hour line angle in degrees - for printing only
                    (setq hrland ( / hrlan 2))
                    (setq hrland ( / hrland pi))
                    (setq hrland ( * hrland 360))
                    (princ "  hour line angle[deg]")    (princ hrland)     (princ " \n")

      
                    ;                    
                    ; ABOVE FIGURES have been validated 
                    ;                    for 32.57 lat, 108.2 lng, ref 105, dec = +- 45


                    ; now we need to convert the angle in radians >>> hrlan
                    ; to coordinates for line drawing, with dial center at >>> x=250 y=50
                    (setq Pt0 '(250 250))
                    ; i.e.      cx  cy

                    (setq hrlan (* +1 hrlan))

                    (setq rotation (- hrlan 1.571))
                    ; 1.570796 is 90 degrees in radians, and LISP here needs a 90 shift
                    (setq pt1  (polar Pt0(+ rotation) (* 200)))

                    ; dont show hour lines off the plate area
                    (setq vhr ctr)
                    ( if(> lha -90)
                        (progn
                        ( if(< lha 90)
                            (progn
                                 (command "pline" Pt0 Pt1 "")  
                                 (command "text" pt1  8   0.0 vhr )            
                            )
                        )
                        )
                    )
                ; ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~ ~
                ) ; end progn for if flag=1
                ) ; end if flag = 1
             
                ; =======================
                ; BUMP HOUR FOR NEXT LOOP
                ; =======================
                ; for ease of reading separate out noon
                ( if(= ctr 12)
                  (princ " \n")
                )
                (setq ctr(+ ctr 1)) 
                (princ "\n")             
         ) 

         (princ "\n")

         ;
         ; AFTER PROGRAM CLEAN UP WITH SD AND SH
         ;
         ; we repeat the SD and SH data ~ however we have different signs
         ; because we reversed the sign earlier, hence why this code is
         ; opposite from the earlier similar code
         ( if (> dec 0)
              (progn
                 (princ "\n...SW *\  gnomon is to right:east   ")
                 (princ "SD: ") (princ sdd) (princ " SH:") (princ shd)
                 (princ "\n")
              )
         )
         ( if (< dec 0)
              (progn
                 (princ "\n...SE /*  gnomon is to left:west   ")
                 (princ "SD: ") (princ sdd) (princ " SH:") (princ shd)
                 (princ "\n")
              )
         )
         ; show the style distance
         (setq Pt0 '(250 250))
         (setq rotation (- sdr 1.571))
         ; 1.570796 is 90 degrees in radians, and LISP here needs a 90 shift
         (setq pt1  (polar Pt0(+ rotation) (* 150)))
         (command "pline" Pt0 Pt1 "")  
         (command "text" pt1  8   0.0 "SD" )
         ; and a style also 
         (setq Pt0 '(250 250))
         (setq sdsh (+ sdr shr))                  ; SE default
         (if (> dec 0)
             (progn 
              (setq sdsh (- sdr shr))             ; SW special code
             )
         )
         (setq rotation (- sdsh 1.571))
         ; 1.570796 is 90 degrees in radians, and LISP here needs a 90 shift
         (setq pt1  (polar Pt0(+ rotation) (* 100)))
         (command "pline" Pt0 Pt1 "")  
         (command "text" pt1  8   0.0 "SH" )

         ;
         ; FINALLY CAUTION ON OUT OF ORDER HOUR LINES
         ;
         (princ "\n*** NOTE *** due to trig, some hour lines may need to be reciprocated, ")
         (princ "you can tell by the jump back/fwd in its labelled HR.\n")

       )
  )

  (princ)

)  
